@@ -3,6 +3,6 @@ TESTNAME=basic-pluto-01
 source /testing/pluto/bin/eastlocal.sh
 
 ipsec setup start
-ipsec auto --add westnet-eastnet
+ipsec auto --add westnet-eastnet-ipv4
 ipsec whack --debug-control --debug-controlmore --debug-crypt
 /testing/pluto/bin/wait-until-pluto-started
