@@ -11,7 +11,7 @@ ping -n -c 4 192.0.2.254
 
 ipsec setup start
 /testing/pluto/bin/wait-until-pluto-started
-ipsec auto --add westnet-eastnet
+ipsec auto --add westnet-eastnet-ipv4
 ipsec whack --debug-control --debug-controlmore --debug-parsing --debug-crypt
 echo done
 
